{*************************************************************}
{            INetDetect   Component for Delphi 32             }
{ Version:   1.0                                              }
{ E-Mail:    mark@francomedia.qc.ca                           }
{ Created:   30 Sep 2000                                      }
{*************************************************************}
{ This component determines online status of the computer     }
{ (Connected or Disconnected).                                }
{*************************************************************}

(*

This component will make the difference between lan and RAS
connection.  All the component i did use before told me i was
connected and return the IP of my network card.  In fact i was
not connected to the internet, but on my local network.

This component, will get all the local network IP.  When it
check for online status, it scan all IP for the local computer
if a IP is detect and this IP is not in the list of local
network card then we have an internet connection.

The only downside is that it can't detect cable modem.  Since
cable modem use an network card the valid IP is in the local
IP list.  You can ask your software user if he as a cable modem
and if the answer is yes you can make a DNS query to verify if
the user is really online.

If you improve this unit send a copy to mark@francomedia.qc.ca

Regards,

Mark Gauthier.
Montreal, Canada.

*)
unit INetDetect;

interface

uses
  Windows, Messages, Classes, Forms, Winsock, ExtCtrls;

type
  TAddrItem = class(TCollectionItem)
  public
   s: string;
  end;

  TINetDetect = class(TComponent)
  private
    FLocalAddrs: TCollection;
    FCurrentIP: String;
    FHostName: PChar;
    FOnline: Boolean;
    FEnabled: Boolean;
    FDispatchInterval: Cardinal;
    FOnChanged: TNotifyEvent;
    Timer: TTimer;
    function  Get(Index: Integer): string;
    function  GetCount: Integer;
    procedure GetLocalIPs;
    procedure SetEnabled(Value: Boolean);
    procedure SetDispatchInterval(Value: Cardinal);
    procedure OnTimer(Sender: TObject);
    procedure DoNothingBool(b: boolean);
    procedure DoNothingString(s: string);
    function  GetFHostName: string;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property LocalIPs[Index: Integer]: string read Get;
    property LocalIPsCount: integer read GetCount;
  published
    property Enabled: Boolean read FEnabled write SetEnabled;
    property DispatchInterval: Cardinal read FDispatchInterval write SetDispatchInterval;
    property Online: Boolean read FOnline write DoNothingBool;
    property CurrentIP : String read FCurrentIP write DoNothingString;
    property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
    property HostName: String read GetFHostName write DoNothingString;
  end;

//{$EXTERNALSYM WSAIoctl}
function WSAIoctl( hSocket: TSocket; ControlCode:dword;
                   InBuf : Pointer; InBufLen:DWord;
                   OutBuf : Pointer; OutBufLen:DWord;
                   BytesReturned : PDWord;
                   lpOverlapped: POverlapped;
                   lpOverlappedRoutine:pointer) : Integer; stdcall;
//{$EXTERNALSYM WSASocket}
function WSASocket(Family, sType, Protocol : Integer;
                   lpProtocolInfo : Pointer;
                   Group : uint;
                   dwFlags : DWORD): TSocket; stdcall;

function WSAIoctl;          external     'ws2_32.dll' name 'WSAIoctl';
function WSASocket;         external     'ws2_32.dll' name 'WSASocketA';


Type

  _INTERFACE_INFO = record
    iiFlags            : ulong;      //* Type and status of the interface */
    iiAddress          : TSockaddr;  //* Interface address */
    iiBroadcastAddress : TSockaddr;  //* Broadcast address */
    iiNetmask          : TSockaddr;  //* Network mask */
  end;

Const
 IFF_UP =		$00000001; //* Interface is up */
 IFF_BROADCAST =	$00000002; //* Broadcast is  supported */
 IFF_LOOPBACK = 	$00000004; //* this is loopback interface */
 IFF_POINTTOPOINT =     $00000008; //*this is point-to-point interface*/
 IFF_MULTICAST =	$00000010; //* multicast is supported */



procedure Register;

implementation

uses sysutils;

procedure TINetDetect.DoNothingString(s: string); begin {} end;

procedure TINetDetect.DoNothingBool(b: boolean); begin {} end;

function  TINetDetect.GetFHostName: string;
begin
     Result := StrPas(FHostName);
end;

constructor TINetDetect.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled := False;
  FDispatchInterval := 1000;

  Getmem(FHostName, 64);
  FLocalAddrs := TCollection.Create(TAddrItem);

  Timer := TTimer.Create(self);
  Timer.Enabled := FEnabled;
  Timer.interval := FDispatchInterval;
  Timer.OnTimer := OnTimer;

  GetLocalIPs;
  OnTimer(Self); // Get current status
end;

destructor TINetDetect.Destroy;
begin
  FLocalAddrs.Clear;
  FLocalAddrs.Free;
  Timer.Free;
  FreeMem(FHostName);

  FEnabled := False;
  inherited Destroy;
end;

procedure TINetDetect.GetLocalIPs;
  const SIO_GET_INTERFACE_LIST : dword = 1074033791;
var
    pAddrInet : TSockAddr;
    OutBufLen,
    RecvBytes,
    i         : DWORD;
    wsError   : Integer;
    WSAData   : TWSAData;
    MySocket  : TSocket;
    localAddr : Array[1..10] of _INTERFACE_INFO; //up to 10 NICs
begin
  wsError := Winsock.WSAStartup(2,  WSAData); //winsock 2 or better.
  If wsError <> 0 then  exit;

  Gethostname(FHostName, 64);
  MySocket := WSASocket(AF_INET, Sock_DGRAM, IPPROTO_UDP, nil,0,0);
  If MySocket = INVALID_SOCKET then exit;
  OutBufLen := Sizeof(localAddr);
  RecvBytes := OutBufLen;
  FillChar(LocalAddr,OutBufLen,0);
  wsError := WSAIoctl(MySocket,SIO_GET_INTERFACE_LIST,nil,0,@localAddr,OutBufLen,@RecvBytes,nil,nil);
  if wsError = SOCKET_ERROR then
  begin
    wsError := WSAGetLastError;
    Raise Exception('Socket error:'+IntToStr(wsError));
    exit;
  end;
  for i := 1 to RecvBytes div Sizeof(_Interface_Info) do
  begin
      pAddrInet := localAddr[i].iiAddress;
      With FLocalAddrs.Add as TAddrItem do s:=inet_ntoa(pAddrInet.sin_addr );
  end;
  closesocket(MySocket);
  WSACleanup();
end;


procedure TINetDetect.OnTimer(Sender: TObject);
type
  TaPInAddr = Array[0..10] of PInAddr;
  PaPInAddr = ^TaPInAddr;
var
  phe: PHostEnt;
  pptr: PaPInAddr;
  I,J: Integer;
  GInitData: TWSAData;
  NewStatus,
  OldStatus : Boolean;
  NewIP: string;
  IP: string;
  Found: Boolean;
begin
  Timer.Enabled := false;
  WSAStartup($101, GInitData);
  GetHostName(FHostName, 64);
  phe := GetHostByName(FHostName);
  if phe = nil then Exit;
  pPtr := PaPInAddr(phe^.h_addr_list);
  I := 0;

  NewIP := '';
  NewStatus := False;
  OldStatus := FOnline;

  while pPtr^[I] <> nil do
   begin
    IP := inet_ntoa(pptr^[I]^);
    Found := false;
    For j := 0 to LocalIPsCount-1 do
    begin
      If (FLocalAddrs.Items[j] as TAddrItem).s = IP then
      begin
           Found := true;
           Break; { Break for }
      end;
    end;
    If Not Found then
    begin
         // If IP is a new Address where are connected...
           NewStatus := true;
           NewIP := IP;
           Break; { break while }
    end;
    Inc(I);
   end;

  WSACleanup;

  FCurrentIP := NewIP;
  If NewStatus <> OldStatus then
  begin
       FOnline := NewStatus;
       if Assigned(FOnChanged) then FOnChanged(self);
  end;

  Timer.Enabled := true;
end;

procedure TINetDetect.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
   begin
    FEnabled := Value;
    Timer.Enabled := Value;
   end;
end;

procedure TINetDetect.SetDispatchInterval(Value: Cardinal);
begin
  if Value <> FDispatchInterval then
   begin
    FDispatchInterval := Value;
    Timer.Interval := Value;
   end;
end;


function TINetDetect.Get(Index: Integer): string;
begin
     If (FLocalAddrs.Count > 0) and (Index In[0..FLocalAddrs.Count]) then
        Result := (FLocalAddrs.Items[Index] as TAddrItem).S ;
end;

function  TINetDetect.GetCount: Integer;
begin
     Result := FLocalAddrs.Count;
end;

procedure Register;
begin
  RegisterComponents('Internet', [TINetDetect]);
end;

end.

